#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import QUB
from qubtree import Node
import CurveFitting
from math import *

# implements CurveFitting module api

class LorentzNLFitter(CurveFitting.NLFitter):
    def __init__(self):
        CurveFitting.NLFitter.__init__(self)
        self.initParams( 3,
                         ('Amp', 1.0, 1.0e-20, 1.0e20),
                         ('X0', 1.0, -1.0e20, 1.0e20),
                         ('Gamma', 1.0, 1.0e-20, 1.0e20),
                         ('Area', 0.0) )
        
    def getEqn(self):
        return '(Amp / PI) * (Gamma / 2) / ( (x - X0)^2 + (Gamma / 2)^2 )'

    def func(self, paramvals, x):
        xdev = x - paramvals[1]
        halfGamma = paramvals[2] / 2
        
        return (paramvals[0] / pi) * halfGamma / (xdev * xdev + halfGamma * halfGamma)

    def deriv(self, paramvals, x):
        xdev = x - paramvals[1]
        halfGamma = paramvals[2] / 2
        denom = xdev * xdev + halfGamma * halfGamma
        result = [0.0] * 3
        result[0] = (1/pi) * halfGamma / denom
        
        denom = denom * denom
        result[1] = (paramvals[0] / pi) * 2 * xdev * halfGamma / denom
        result[2] = (paramvals[0] / (2 * pi)) * (xdev * xdev - halfGamma * halfGamma) / denom
        return result

CurveFitting.fitters['Lorentzian'] = LorentzNLFitter

class Log10LorentzNLFitter(CurveFitting.NLFitter):
    def __init__(self):
        CurveFitting.NLFitter.__init__(self)
        self.initParams( 3,
                         ('Amp', 1.0, 1.0e-20, 1.0e20),
                         ('X0', 1.0, -1.0e20, 1.0e20),
                         ('Gamma', 1.0, 1.0e-20, 1.0e20),
                         ('Area', 0.0) )
        
    def getEqn(self):
        return 'log10 [(Amp / PI) * (Gamma / 2) / ( (x - X0)^2 + (Gamma / 2)^2 )]'

    def func(self, paramvals, x):
        xdev = x - paramvals[1]
        halfGamma = paramvals[2] / 2
        
        return log10((paramvals[0] / pi) * halfGamma / (xdev * xdev + halfGamma * halfGamma))

    def deriv(self, paramvals, x):
        xdev = x - paramvals[1]
        halfGamma = paramvals[2] / 2
        denom = xdev * xdev + halfGamma * halfGamma
        result = [0.0] * 3
        result[0] = 1 / (paramvals[0] * log(10))
        fvalLog10 = log(10) * (paramvals[0] / pi) * halfGamma / (xdev * xdev + halfGamma * halfGamma)
        
        denom = denom * denom
        result[1] = (paramvals[0] / pi) * 2 * xdev * halfGamma / denom / fvalLog10
        result[2] = (paramvals[0] / (2 * pi)) * (xdev * xdev - halfGamma * halfGamma) / denom / fvalLog10
        return result

# CurveFitting.fitters['Lorentzian Log10'] = Log10LorentzNLFitter
